# 01. Algorithm integration

This workflow integrates all variant calls in a batch of samples on a
per-algorithm basis.

Frequently, samples are called jointly in a small group, which is a subset of a
larger batch of sequencing libraries. To accommodate this paradigm, the user
specifies a sample key, indicating which group and batch each sample belongs
to, e.g.

```
sample	group	batch
11194.fa	11194	pcr_plus
11006.mo	11006	pcr_minus
```

The workflow expects a PE/SR VCF for each group, and a depth bed for each
batch, as detailed below, and integrates all variants belonging to a single
batch. A group may be a single sample; simply use the sample ID as the group
ID.

## Input files

Variant calls must be standardized to the following specifications before
integration. See the example preprocessing module for guidance.

* `input_vcfs/{source}.{group}.vcf.gz`  
    Tabix-indexed PE/SR VCFs per algorithm. 
    - {source} indicates the source algorithm.
    - {group} indicates a subgroup of samples which were called jointly.
    - VCF records are required to include INFO fields for SVTYPE, CHR2, END,
      STRANDS [++,+-,-+,--], SVLEN, and SOURCES.  Currently, DEL, DUP, INV, and
      BND are supported SVTYPEs; INS have not been tested. BND and INV
      breakpoints must be segregated and annotated with strand.  
      (NOTE: END has become a reserved attribute in pysam 0.11.2.1 and may be
      deprecated here in the future.)
* `input_beds/{batch}.{svtype}.bed.gz`  
    All per-sample depth calls in a batch, merged across algorithms.
    - Depth calls must be segregated by {svtype}, which may be DEL or DUP.
    - Each line corresponds to a single call in a single sample.
    - First six columns must be chrom, start, end, name, sample, svtype. Any
      following columns are discarded during integration.
    - In our preprocessing module, these files are generated by taking all
      algorithms's calls for a given sample and performing a bedtools merge,
      then concatenating and sorting all sample calls.

## Output files

* `vcfcluster/{batch}.{source}.{chrom}.vcf`  
    Clustered PE/SR variants, per batch, per source, and per chromosome.
* `rdtest_beds/{batch}.{source}.{chrom}.bed`  
    Clustered PE/SR variants in RdTest format. 

## Manual process 

Follow these steps to manually cluster pair-end split read (pesr) calls:

1. Cluster all VCFs in a batch from a given algorithm
```
svtk vcfcluster vcflists/{batch}.{source}.list stdout -r {chrom} -d 300 -f 0.1 -x {blacklist} -z 0 -t {SVtype} | vcf-sort -c | bgzp -c > vcfcluster/{batch}.{source}.{chrom}.vcf.gz 
tabix vcfcluster/{batch}.{source}.{chrom}.vcf.gz
```
note: `SVtype` usually represents `DEL,DUP,INV,BND` for **pesr** caller and `INS` for **MEI** caller

2. Convert to RdTest format
```
python scripts/make_pesr_rdtest_bed.py vcfcluster/{batch}.{source}.{chrom}.vcf.gz rdtest_beds/{batch}.{source}.{chrom}.bed
```


Follow these steps to manually cluster depth calls:

1. Cluter CNV calls

```
svtk bedcluster ../00_preprocessing/std_beds/{batch}.{svtype}.bed.gz -r {chrom} -p {batch}_depth_{svtype}_{chrom} > bedcluster/{batch}.{svtype}.{chrom}.bed
```
2. Aggregate observations into variants and convert to RdTest format
```
cat \
  <(scripts/make_depth_rdtest_bed.py bedcluster/{batch}.DEL.{chrom}.bed | sed '1d') \
  <(scripts/make_depth_rdtest_bed.py bedcluster/{batch}.DUP.{chrom}.bed | sed '1d') \
  | sort -k1,1V -k2,2n \
  | cat <(echo -e "#chrom start end name samples svtype" | sed -e 's/ /\\t/g') - \
  > rdtest_beds/{batch}.depth.{chrom}.bed
```
3. Transform rdtest_beds to vcf
```
svtk rdtest2vcf rdtest_beds/{batch}.depth.{chrom}.bed ref/batches.list vcfcluster/{batch}.depth.{chrom}.vcf.gz
```
